package com.pmease.quickbuild.plugin.tracker.redmine;

import java.util.Map;

import org.hibernate.validator.constraints.NotEmpty;

import com.pmease.quickbuild.Context;
import com.pmease.quickbuild.annotation.Editable;
import com.pmease.quickbuild.annotation.Multiline;
import com.pmease.quickbuild.annotation.ScriptApi;
import com.pmease.quickbuild.annotation.Scriptable;
import com.pmease.quickbuild.plugin.tracker.core.extensionpoint.Issue;
import com.pmease.quickbuild.plugin.tracker.core.step.AbstractIssueStep;
import com.pmease.quickbuild.plugin.tracker.core.utils.StringHelper;

public class CreateIssueStep extends AbstractIssueStep {

	private static final long serialVersionUID = 1L;

	private String issueSummary;
	private String issueDescription;
	private String issuePriority;
	private String issueType;
	
	@Override
	public void run() {
		onBeforeRun();
		
		RedmineCli cli = RedmineCli.newInstance();

		String extraOptions = getExtraAttributes();
		Map<String, String> map = StringHelper.getOptionsFromDefinition(extraOptions);
		map.put("subject", getIssueSummary());
		map.put("description", getIssueDescription());
		//map.put("priority_id", getPriorityId());
		//map.put("tracker_id", getTrackerId());
		
		Issue issue = cli.createIssue(map);
		
		getIssueActions().put(issue.getIssueKey(), "create");
		Context.getLogger().info("Issue: " + issue + " created successfully.");
		onAfterRun();
	}

	@Editable(name = "Issue Subject", order = 1000)
	@NotEmpty
	@Scriptable
	@ScriptApi("Get issue summary")
	public String getIssueSummary() {
		return issueSummary;
	}

	public void setIssueSummary(String issueSummary) {
		this.issueSummary = issueSummary;
	}

	@Editable(name="Issue Description", order=1100)
	@Scriptable
	@ScriptApi("Get issue description")
	public String getIssueDescription() {
		return issueDescription;
	}

	public void setIssueDescription(String issueDescription) {
		this.issueDescription = issueDescription;
	}

	@Editable(name="Issue Priority", order=1300)
	@ScriptApi("Get issue priority")
	public String getIssuePriority() {
		return issuePriority;
	}

	public void setIssuePriority(String issuePriority) {
		this.issuePriority = issuePriority;
	}

	@Editable(name="Tracker Type", order=1200)
	@ScriptApi("Get issue tracker type")
	public String getIssueType() {
		return issueType;
	}

	public void setIssueType(String issueType) {
		this.issueType = issueType;
	}
	
	@Override
	@Editable(order=3000, name="Extra Attributes", description="Specify extra attributes of the issue, " +
			"with one attribute per line. For example:<br/>" +
			"<code>field_1=value1<br/>" +
			"field2=value2</code>" +
			"Consult your issue tracker administrator for available attributes.")
	@Multiline
	@Scriptable
	@ScriptApi("Get extra attributes which will be used to create issue.")
	public String getExtraAttributes() {
		return super.getExtraAttributes();
	}
}
